
@echo off
setlocal enableextensions enabledelayedexpansion


rem ###########################################################################################
rem ###########################################################################################
rem ###################                                                    ####################
rem ###################      New Zealand Sites ZIP Run Batch File      ####################
rem ###################                                                    ####################
rem ###########################################################################################
rem ###########################################################################################



rem ===========================================================================================
rem                       :SETUP 
rem ===========================================================================================
rem   This section sets up initial variables, reads the INI file for site specific parameters,
rem   sets the log file variables, and displays the header.
rem ===========================================================================================

:SETUP
	rem --------------------- trigger the delayed expansion functionality
	for /f %%a in ("%~0") do (echo %%a)
	
	rem --------------------- Variable to be able to use %bat_return% to return from subroutines.
	set bat_return=goto :EOF

	rem --------------------- bat_exedir variable is where the rundir should be.
	set bat_exedir=%~dp0::
	set bat_exedir=!bat_exedir:\::=!
	set bat_exedir=!bat_exedir:::=!

	rem -------------------------------------------------------------------------------------------
	rem   Read the run parameters for processing at a given site
	rem 
	rem       First camera number of a 16 camera set with sequential/inclusive numbering
	rem       Number of independent detection processes to run during capture
	rem       Start and stop time is based on sun elevation angle in deg below local horizon
	rem       Number of FF files to skip for any given camera when doing star extraction
	rem       Number of stars to extract from each FF file
	rem       Number of CapturedFiles subfolders to retain before purging (deleting for space)
	rem       The CAMS directory full pathname up to and WITHOUT the \CAMS appended to the end
	rem
	rem   Note that "ReadParams_capture16.bat" reads the "capture16.params.ini" file to
	rem        retrieve all the variables for the above parameters. 
	rem -------------------------------------------------------------------------------------------
 
	echo.[!date:~4! !time!]  call "!bat_exedir!\ReadParams_capture16.bat"
	                         call "!bat_exedir!\ReadParams_capture16.bat"


	rem --------------------- bat_LogFile is the target file for daily logging of processes run

	set bat_LogFile=!bat_CAMS_distrofolder!\DailyRunLog_!bat_firstcamera!.txt
	set bat_batchfile=%~nx0&	rem bat_batchfile is used for identifying which subroutine in the logs


	rem --------------------- Indicate the start of the script on the monitor and in the running log file 
	echo. bat_LogFile=!bat_LogFile!
	echo.>>"!bat_LogFile!"
	echo.>>"!bat_LogFile!"
    	echo.[!date:~4! !time!]  ================================================
    	echo.[!date:~4! !time!]  =================================================================================>>"!bat_LogFile!"
	echo.[!date:~4! !time!]  %~nx0 %*
	echo.[!date:~4! !time!]  BATCH %~nx0 %*>>"!bat_LogFile!"

	set bat_capturesession=%~1

	if /i "!bat_capturesession!" equ "" (
		echo. ### ERROR - no capture session directory provided ERROR ###
		goto :HELP
	)



	rem ---------------------  See :SIM_DELETE for description of simulation
	rem                        Only perform the simulation deletion subroutine if simulation is enabled

	if /i "!bat_SIMULATION.simulate!" equ "true" (
		call :SIM_DELETE
	)

	goto :MAIN %*




rem ===============================================================================================
rem                                 :MAIN 
rem ===============================================================================================
rem   This is the main section of code that calls all the other processing subroutines in the
rem   following work flow order:
rem
rem      Delete the oldest capture folders that exceed the user specified max # of capture folders
rem      Delete the oldest FF backup folders that exceed the user specified max # of backup folders
rem      Obtain the most recent (previous collection night) capture folder
rem      Launch the 16 channel capture and detection triggered by user specified sun elevation
rem      Obtain the new capture folder just populated
rem      Combine the multiple detectinfo files from the multi-core detection processing
rem      Run star extraction on a user specified subset of FF files across all cameras
rem      Zip the Detectinfo and CALSTARS files in the DistroFiles folder
rem      FTP the zipped file by uploading to multiple sites
rem       
rem ===============================================================================================

:MAIN


	call :COMBINE_DETECTINFO %*

			if !errorlevel! neq 0 ( goto :MAIN_END )


	call :STAR_EXTRACTION %*

			if !errorlevel! neq 0 ( goto :MAIN_END )


	call :ZIP_DISTROTXTFILES %*

			if !errorlevel! neq 0 ( goto :MAIN_END )


   	call :FTP_UPLOAD %*

			if !errorlevel! neq 0 (
				rem ----- The return code from this is compound. The thousands digits 
				rem       are the return code from the first transfer and the ones 
				rem       digits are the return code from the second transfer. 
				rem       It doesn't matter at this point which one fails unless  
				rem       there is something else to add after this section.
				goto :MAIN_END
			)


:MAIN_END

	goto :DONE




Rem ===========================================================================================
Rem                         :DEL_OLDCAPTURE
Rem ===========================================================================================
Rem   Identify the older CapturedFiles folders by sorting in reverse name order and then
Rem   delete the oldest CaptureFiles folders if there exist more than the subfolder limit.
Rem ===========================================================================================

:DEL_OLDCAPTURE

	echo.[!Date:~4! !Time!]      CALL   DEL_OLDCAPTURE %*>>"!bat_LogFile!"

        set bat_returncode=0

	set bat_subfolder_count=0

	for /f "usebackq tokens=*" %%a in (`dir "!bat_CAMS_capturefolder!\20??_*" /ad /b /o-n`) do (

		set bat_subfolder_oldcapture=%%~na

		set /a bat_subfolder_count+=1

		if !bat_subfolder_count! GTR !bat_captured_folder_limit!  (

			echo.[!Date:~4! !Time!]          Deleting files in  "!bat_CAMS_capturefolder!\!bat_subfolder_oldcapture!">>"!bat_LogFile!"
			                                 del   /s /q "!bat_CAMS_capturefolder!\!bat_subfolder_oldcapture!"

			echo.[!Date:~4! !Time!]          Deleting directory "!bat_CAMS_capturefolder!\!bat_subfolder_oldcapture!">>"!bat_LogFile!"
			                                 rmdir /s /q "!bat_CAMS_capturefolder!\!bat_subfolder_oldcapture!"


			if exist "!bat_CAMS_capturefolder!\!bat_subfolder_oldcapture!" (
				echo.[!Date:~4! !Time!] ###ERROR - FAILED TO REMOVE FOLDER: "!bat_CAMS_capturefolder!\!bat_subfolder_oldcapture! ###ERROR">>"!bat_LogFile!"
				set bat_returncode=1
			)
		)
	)


:DEL_OLDCAPTURE_END

	echo.[!Date:~4! !Time!]      RETURN DEL_OLDCAPTURE         [exitcode=!bat_returncode!, FF capture folder count=!bat_subfolder_count!]>>"!bat_LogFile!"
	exit /b !bat_returncode!






Rem ===========================================================================================
Rem                         :DEL_OLDFFBACKUP
Rem ===========================================================================================
Rem   Identify the older CapturedFiles folders by sorting in reverse name order and then
Rem   delete the oldest CaptureFiles folders if there exist more than the subfolder limit.
Rem ===========================================================================================

:DEL_OLDFFBACKUP

	echo.[!Date:~4! !Time!]      CALL   DEL_OLDFFBACKUP %*>>"!bat_LogFile!"

	set bat_subfolder_count=0

	for /f "usebackq tokens=*" %%a in (`dir "!bat_CAMS_ffbackupfolder!\20??_*" /ad /b /o-n`) do (

		set bat_subfolder_oldcapture=%%~na

		set /a bat_subfolder_count+=1

		if !bat_subfolder_count! GTR !bat_ff_backup_folder_limit!  (

			echo.[!Date:~4! !Time!]          Deleting files in  "!bat_CAMS_ffbackupfolder!\!bat_subfolder_oldcapture!">>"!bat_LogFile!"
			                                 del   /s /q "!bat_CAMS_ffbackupfolder!\!bat_subfolder_oldcapture!"
 
			echo.[!Date:~4! !Time!]          Deleting directory "!bat_CAMS_ffbackupfolder!\!bat_subfolder_oldcapture!">>"!bat_LogFile!"
			                                 rmdir /s q "!bat_CAMS_ffbackupfolder!\!bat_subfolder_oldcapture!"

			if exist "!bat_CAMS_ffbackupfolder!\!bat_subfolder_oldcapture!" (
				echo.[!Date:~4! !Time!] ###ERROR - FAILED TO REMOVE FOLDER: "!bat_CAMS_ffbackupfolder!\!bat_subfolder_oldcapture! ###ERROR">>"!bat_LogFile!"
				set bat_returncode=1
			)
		)
	)


:DEL_OLDFFBACKUP_END

	echo.[!Date:~4! !Time!]      RETURN DEL_OLDFFBACKUP        [exitcode=!bat_returncode!, FF backups folder count=!bat_subfolder_count!]>>"!bat_LogFile!"
	exit /b !bat_returncode!





Rem ===========================================================================================
Rem                     :GET_PRECAPTUREFOLDER
Rem ===========================================================================================
Rem   Identify the most recently run CapturedFiles subfolder by sorting on folder name (date)
Rem   from earliest created to most recent.
Rem ===========================================================================================

:GET_PRECAPTUREFOLDER

	echo.[!Date:~4! !Time!]      CALL   GET_PRECAPTUREFOLDER %*>>"!bat_LogFile!"

	set bat_returncode=0

	set bat_subfolder_precapture=2000_01_01_00_00_00


	for /f "usebackq tokens=*" %%a in (`dir "!bat_CAMS_capturefolder!\20??_*" /ad /b /on`) do (

		set bat_subfolder_precapture=%%~na
	)
	
	if /i "!bat_subfolder_precapture!" equ "2000_01_01_00_00_00" (
		echo.[!Date:~4! !Time!]          No previous directories in the CapturedFiles directory. >>"!bat_LogFile!"
	)


:GET_PRECAPTUREFOLDER_END

	echo.[!Date:~4! !Time!]      RETURN GET_PRECAPTUREFOLDER   [exitcode=!bat_returncode!, Previous capture \!bat_subfolder_precapture!]>>"!bat_LogFile!"
	exit /b !bat_returncode!


	


Rem ===========================================================================================
Rem                     :LAUNCH_CAPTURE16DETECT
Rem ===========================================================================================
Rem   Launch the real-time 16-channel capture and detection code that runs as a multi-threaded
Rem   process for capture, compression into FF files, and also spawns child processes for the 
Rem   MeteorScan detection algorithm (user specified number of child processes each running
Rem   round robin on a single FF file at a time). Note that for each child process assigned, 
Rem   an independent Detectinfo file is generated that will get merged later.
Rem
Rem   In addition, the Capture16 program also saves off a single FF file for each camera at
Rem   a time midway through the night's collection and places that in the FFbackupFiles folder.
Rem   Subfolders are all named for the starting date and time of the collection in UT as:
Rem
Rem                   ....\YYYY_MM_DD_HH_MM_SS\FF*.bin
Rem
Rem   args = 1st camera, #detect processes, sun angle below horizon for capture, CAMS directory
Rem 
Rem ============================================================================================

:LAUNCH_CAPTURE16DETECT
 
	echo.[!Date:~4! !Time!]      CALL   LAUNCH_CAPTURE16DETECT %*>>"!bat_LogFile!"

	if /i "!bat_SIMULATION.simulate!" equ "true" (
		call :SIM_CAPTURE
		set bat_returncode=0
		goto :LAUNCH_CAPTURE16DETECT_END
	)

	echo.[!date:~4! !time!]      CALL  "!bat_CAMS_runfolder!\FTP_Capture16AndDetect.exe"  !bat_firstcamera!  !bat_numdetectproc!  !bat_sun_elevation_deg!  "!bat_CAMS_directory!">>"!bat_LogFile!"
	                             CALL  "!bat_CAMS_runfolder!\FTP_Capture16AndDetect.exe"  !bat_firstcamera!  !bat_numdetectproc!  !bat_sun_elevation_deg!  "!bat_CAMS_directory!"
	
	set bat_returncode=!errorlevel!


:LAUNCH_CAPTURE16DETECT_END

	echo.[!Date:~4! !Time!]      RETURN LAUNCH_CAPTURE16DETECT [exitcode=!bat_returncode!]>>"!bat_LogFile!"
	exit /b !bat_returncode!




Rem ===========================================================================================
Rem                     :GET_POSTCAPTUREFOLDER
Rem ===========================================================================================
Rem   Identify newly generated CapturedFiles subfolder by sorting on folder name (date) from
Rem   earliest created to most recent.
Rem ===========================================================================================

:GET_POSTCAPTUREFOLDER

	echo.[!Date:~4! !Time!]      CALL   GET_POSTCAPTUREFOLDER %*>>"!bat_LogFile!"

	set bat_returncode=1
	set bat_capturesession=2000_01_01_00_00_00

	for /f "usebackq tokens=*" %%a in (`dir "!bat_CAMS_capturefolder!\20??_*" /ad /b /on`) do (

		set bat_capturesession=%%~na
		set bat_returncode=0
	)

	if "!bat_returncode!" neq "0" (
		echo.[!Date:~4! !Time!]     ###ERROR - No New Captured Files Folder ###ERROR>>"!bat_LogFile!"
	)


:GET_POSTCAPTUREFOLDER_END

	echo.[!Date:~4! !Time!]      RETURN GET_POSTCAPTUREFOLDER  [exitcode=!bat_returncode!, Capture folder = !bat_capturesession!]>>"!bat_LogFile!"
	exit /b !bat_returncode!





Rem ===========================================================================================
Rem                          :COMBINE_DETECTINFO
Rem ===========================================================================================
Rem
Rem   If a new subfolder of imagery created (a.k.a. new capture session completed) then:
Rem
Rem        Create new subfolder under the distro folder for the given night
Rem        Combine detectinfo files creating FTPdetectinfo_000000_YYYY_MM_DD_HH_MM_SS.txt
Rem        Rename merged detectinfo from 000000 to cam# so other sites can be combined (merged)
Rem        Copy detectinfo and star files to the new subfolder in the distro folder 
Rem
Rem ===========================================================================================

:COMBINE_DETECTINFO

	echo.[!Date:~4! !Time!]      CALL   COMBINE_DETECTINFO %*>>"!bat_LogFile!"

	set bat_returncode=0

	if /i "!bat_subfolder_precapture!" NEQ "!bat_capturesession!"  (

		mkdir "!bat_CAMS_distrofolder!\!bat_capturesession!"

		echo.[!Date:~4! !Time!]      CALL  "!bat_CAMS_runfolder!\FTP_CombineDetectinfo.exe"  "!bat_CAMS_capturefolder!\!bat_capturesession!">>"!bat_LogFile!"
		                             CALL  "!bat_CAMS_runfolder!\FTP_CombineDetectinfo.exe"  "!bat_CAMS_capturefolder!\!bat_capturesession!"

		set bat_returncode=!errorlevel!

		if /i "!bat_returncode!" neq "0" (
			[!Date:~4! !Time!] ###ERROR [!bat_returncode!] - FAILED TO COMBINE_DETECTINFO: "!bat_CAMS_capturefolder!\!bat_capturesession! ###ERROR">>"!bat_LogFile!"
			goto :COMBINE_DETECTINFO_END
		)

		set bat_detectinfo_filename=

		for /f "usebackq tokens=*" %%a in (`dir "!bat_CAMS_capturefolder!\!bat_capturesession!\FTPdetectinfo_000000_*.txt" /a-d /b /on`) do (

			set bat_detectinfo_filename=%%~na



                               if !bat_firstcamera! leq 9    (
                                                                set bat_detectinfo_filename_cam=!bat_detectinfo_filename:_0000_=_000%bat_firstcamera%_!
                        ) else (
                               if !bat_firstcamera! leq 99   (
                                                                set bat_detectinfo_filename_cam=!bat_detectinfo_filename:_0000_=_00%bat_firstcamera%_!
                        ) else (
                               if !bat_firstcamera! leq 999  (
                                                                set bat_detectinfo_filename_cam=!bat_detectinfo_filename:_0000_=_0%bat_firstcamera%_!
                        ) else (
                               if !bat_firstcamera! leq 9999 (
                                                                set bat_detectinfo_filename_cam=!bat_detectinfo_filename:_0000_=_%bat_firstcamera%_!
                        ) else (
                               if !bat_firstcamera! leq 99999 (
                                                                set bat_detectinfo_filename_cam=!bat_detectinfo_filename:_00000_=_%bat_firstcamera%_!
                        ) else (
                               if !bat_firstcamera! leq 999999 (
                                                                set bat_detectinfo_filename_cam=!bat_detectinfo_filename:_000000_=_%bat_firstcamera%_!
                        )
                        )
                        )
                        )
                        )
                        )


			copy /y "!bat_CAMS_capturefolder!\!bat_capturesession!\!bat_detectinfo_filename!.txt" "!bat_CAMS_capturefolder!\!bat_capturesession!\!bat_detectinfo_filename_cam!.txt"

			copy /y "!bat_CAMS_capturefolder!\!bat_capturesession!\!bat_detectinfo_filename_cam!.txt" "!bat_CAMS_distrofolder!\!bat_capturesession!\!bat_detectinfo_filename_cam!.txt"

			if exist "!bat_CAMS_distrofolder!\!bat_capturesession!\!bat_detectinfo_filename_cam!.txt" (
				echo.[!Date:~4! !Time!]          Detectinfo File Copied to Distro "!bat_CAMS_distrofolder!\!bat_capturesession!\!bat_detectinfo_filename_cam!.txt">>"!bat_LogFile!"
			) else (
				set bat_returncode=1
				echo.[!Date:~4! !Time!] ###ERROR - Failure to copy Detectinfo to Distro "!bat_CAMS_distrofolder!\!bat_capturesession!\!bat_detectinfo_filename_cam!.txt" ###ERROR>>"!bat_LogFile!"
			)
		)
		
		if not defined bat_detectinfo_filename (
			set bat_returncode=2
			echo.[!Date:~4! !Time!] ###ERROR - Unable to perform loop: [for ... dir "!bat_CAMS_capturefolder!\!bat_capturesession!\FTPdetectinfo_000000_*.txt" /a-d /b /on] ###ERROR>>"!bat_LogFile!"
		)

	) else (
		set bat_returncode=3
		echo.[!Date:~4! !Time!] ###ERROR - No new CapturedFiles folder. Skipping post-capture processing ###ERROR>>"!bat_LogFile!"
	)


:COMBINE_DETECTINFO_END

	echo.[!Date:~4! !Time!]      RETURN COMBINE_DETECTINFO     [exitcode=!bat_returncode!]>>"!bat_LogFile!"
	exit /b !bat_returncode!





Rem ===============================================================================================
Rem                             :STAR_EXTRACTION
Rem ===============================================================================================
Rem    This subroutine calls the FTP_CalStarExtractor.exe program using the command line arguments
Rem    specific to the captured files folder (a.k.a. post capture session). This will extract and
Rem    save in a text file for each camera, the extracted star centroids from a sampling of FF  
Rem    files over the course of the night's collection. The CALSTARS*.txt files will be copied to
Rem    a distribution folder.
Rem 
Rem    args = #FF files to step over, #stars to extract, FF file directory
Rem ===============================================================================================

:STAR_EXTRACTION

	echo.[!Date:~4! !Time!]      CALL   :STAR_EXTRACTION %*>>"!bat_LogFile!"

	set bat_returncode=0

	echo.[!Date:~4! !Time!]      CALL  "!bat_CAMS_runfolder!\FTP_CalStarExtractor.exe"  !bat_nstep_FFfile!  !bat_nstars!  "!bat_CAMS_capturefolder!\!bat_capturesession!">>"!bat_LogFile!"
	                             CALL  "!bat_CAMS_runfolder!\FTP_CalStarExtractor.exe"  !bat_nstep_FFfile!  !bat_nstars!  "!bat_CAMS_capturefolder!\!bat_capturesession!"

	set bat_returncode=!errorlevel!


        rem -------------------------- Error checking and if success then move CALSTARS files to Distro folder
		  
	if !bat_returncode! EQU 3  (
		echo.[!Date:~4! !Time!] ###ERROR Star Extractor failed to find dir list ###ERROR>>"!bat_LogFile!" 
		goto :STAR_EXTRACTION_END
	)

	if !bat_returncode! EQU 4  (
		echo.[!Date:~4! !Time!] ###ERROR Star Extractor found no FF files       ###ERROR>>"!bat_LogFile!" 
		goto :STAR_EXTRACTION_END
	)

	if !bat_returncode! EQU 0  (

		echo.[!Date:~4! !Time!]          Finished extracting stars >>"!bat_LogFile!"

		rem ----------------------- Copy all "CALSTARS*.txt" files from CapturedFiles folder to DistroFiles folder.

		set bat_star_filename=

		for /f "usebackq tokens=*" %%a in (`dir "!bat_CAMS_capturefolder!\!bat_capturesession!\CALSTARS*.txt" /a-d /b /on`) do (

			set bat_star_filename=%%~na

			copy /y "!bat_CAMS_capturefolder!\!bat_capturesession!\!bat_star_filename!.txt" "!bat_CAMS_distrofolder!\!bat_capturesession!"
		)

		if not defined bat_star_filename (

			rem ------------ You get here if the for loop didn't start because the dir command in the parenthesis failed.

			echo.[!Date:~4! !Time!] ###ERROR failed to loop [for... dir "!bat_CAMS_capturefolder!\!bat_capturesession!\CALSTARS*.txt" /a-d /b /on] ###ERROR>>"!bat_LogFile!"

			goto :STAR_EXTRACTION_END
		)

		echo.[!Date:~4! !Time!]          Cal star centroid files copied to Distro "!bat_CAMS_distrofolder!\!bat_capturesession!" >>"!bat_LogFile!"

	) else (
		echo.[!Date:~4! !Time!] ###ERROR Star Extractor had some undefined error = !bat_returncode!   ###ERROR>>"!bat_LogFile!" 
		set bat_returncode=99
		goto :STAR_EXTRACTION_END
	)


:STAR_EXTRACTION_END

	echo.[!Date:~4! !Time!]      RETURN :STAR_EXTRACTION        [exitcode=!bat_returncode!]>>"!bat_LogFile!"
	exit /b !bat_returncode!



Rem ===============================================================================================
Rem                           :ZIP_DISTROTXTFILES
Rem ===============================================================================================
Rem   This subroutine zips the results in the distro files folder by calling the 7za.exe tool.
Rem   This is to prepare for transfer of the relevent files (Detectinfo and CAMSTARS*) to the 
Rem   central processing center. 
Rem
Rem   The bat_zipfile variable contains the full path of the zip file.
Rem   The bat_zip variable contains just the filename without the path.
Rem
Rem ===============================================================================================

:ZIP_DISTROTXTFILES

	echo.[!Date:~4! !Time!]      CALL   :ZIP_DISTROTXTFILES %*>>"!bat_LogFile!"

	set bat_returncode=0
	
	se ssion!_!bat_firstcamera!.zip

	set bat_zip=!bat_capturesession!_!bat_firstcamera!.zip

	echo.[!Date:~4! !Time!]          Running "!bat_CAMS_runfolder!\7za.exe"  a  -tzip  "!bat_CAMS_distrofolder!\!bat_capturesession!_!bat_firstcamera!.zip" "!bat_CAMS_distrofolder!\!bat_capturesession!\*.txt">>"!bat_LogFile!
	                                    call "!bat_CAMS_runfolder!\7za.exe"  a  -tzip  "!bat_CAMS_distrofolder!\!bat_capturesession!_!bat_firstcamera!.zip" "!bat_CAMS_distrofolder!\!bat_capturesession!\*.txt"

	set bat_returncode=!errorlevel!


:ZIP_DISTROTXTFILES_END

	echo.[!Date:~4! !Time!]      RETURN :ZIP_DISTROTXTFILES     [exitcode=!bat_returncode! for 7zip]>>"!bat_LogFile!"
	exit /b !bat_returncode!




rem =======================================================================
rem                            :FTP_UPLOAD
rem =======================================================================
rem   This subroutine calls the ftp transfer routine twice...
rem
rem   The first call with the standard variables from the [FTP] section
rem   of the INI file:  
rem     [FTP]
rem     bat_ftppassword=pjenniskens@seti.org
rem     bat_ftpsite=ftp.seti.org
rem     bat_ftpuser=anonymous
rem     bat_ftpdir=incoming/cams
rem     transfer_script=ftp_transfer_seti.bat
rem
rem   The second time uses the variables from the [FTP2] section.
rem     [FTP2]
rem     ftppassword=
rem     ftpsite=ftp.davesamuels.com
rem     ftpuser=davesamu
rem     ftpdir=web/junk/testftp
rem     transfer_script=ftp_transfer_seti.bat
rem
rem   The "transfer_script" variable allows the user to specify a custom 
rem   script to use for transfer in case the standard transfer won't do the 
rem   job. It is possible that if a custom script is required, it's okay to
rem   add any variables to the FTP2 section that would be required to 
rem   support the transfer, even if a protocol other than FTP is used.
rem
rem   The bat_returncode is a compound return code.  The thousands digit
rem   is the return code from the first transfer and the ones digit is the
rem   return code from the second transfer.
rem =======================================================================

:FTP_UPLOAD

	echo.[!Date:~4! !Time!]      CALL   :FTP_UPLOAD %*>>"!bat_LogFile!"

	set bat_returncode=0

	echo.[!Date:~4! !Time!]      call "!bat_CAMS_runfolder!\!bat_FTP.transfer_script!" "!bat_zipfile!" "!bat_zip!">>"!bat_LogFile!"
	                             call "!bat_CAMS_runfolder!\!bat_FTP.transfer_script!" "!bat_zipfile!" "!bat_zip!"

	set /a bat_returncode=!errorlevel!*1000

	rem -------------------- Save the original variables... kind of like PUSHing them on the stack
 
	SETLOCAL enableextensions enabledelayedexpansion
	
		rem ................ Copy the FTP2 variables to the standard variables

		set bat_ftppassword=!bat_FTP2.ftppassword!
		set bat_ftpsite=!bat_FTP2.ftpsite!
		set bat_ftpuser=!bat_FTP2.ftpuser!
		set bat_ftpdir=!bat_FTP2.ftpdir!

		if not defined bat_ftpsite goto :FTP_UPLOAD2

		rem ................ Now call the transfer routine a second time.  If
 
		echo.[!Date:~4! !Time!]      call "!bat_CAMS_runfolder!\!bat_FTP2.transfer_script!" "!bat_zipfile!" "!bat_zip!">>"!bat_LogFile!"
		                             call "!bat_CAMS_runfolder!\!bat_FTP2.transfer_script!" "!bat_zipfile!" "!bat_zip!"

		set /a bat_returncode += !errorlevel!

	rem -------------------- POP the original FTP parameters

:FTP_UPLOAD2
	rem Use ENDLOCAL to remove the changed variables
	ENDLOCAL


:FTP_UPLOAD_END

	echo.[!Date:~4! !Time!]      RETURN :FTP_UPLOAD             [exitcode=!bat_returncode!]>>"!bat_LogFile!"
	exit /b !bat_returncode!





rem ==========================================================================================
rem   :SIM_DELETE
rem   If simulation is enabled, the first thing to do is to remove the existing
rem   capture directory.  
rem   Simulation is enabled by setting the "simulate=" flag in the [SIMULATION] 
rem   section of the INI file to "true".  For example:
rem      [SIMULATION]
rem      simulate=true
rem      sim_session=2013_10_30_06_21_56
rem      sim_sourcedir=!bat_CAMS_directory!\Simulation\CapturedFiles\2013_10_30_06_21_56
rem 
rem   The "sim_session" variable is the session date/time stamp [The CapturedFiles
rem   folder name].
rem
rem   The "sim_sourcedir" variable is the full path for the :SIM_CAPTURE subroutine
rem   to copy from.  
rem
rem   This :SIM_DELETE performs the function of clearing out the session's
rem   files in order to start fresh.  The :SIM_CAPTURE subroutine will be 
rem   called from within the :LAUNCHCAPTURE subroutine only if "simulate=true".
rem =========================================================================================

:SIM_DELETE

	echo.[!Date:~4! !Time!]      CALL SIM_CAPTURE %*>>"!bat_LogFile!"

	rem -------------------- set bat_SIMULATION.

	echo.[!Date:~4! !Time!]  DEL /Q /s "!bat_CAMS_capturefolder!\!bat_SIMULATION.sim_session!"
	                         DEL /Q /s "!bat_CAMS_capturefolder!\!bat_SIMULATION.sim_session!"
	echo.[!Date:~4! !Time!]  RD  /Q /s "!bat_CAMS_capturefolder!\!bat_SIMULATION.sim_session!"
	                         RD  /Q /s "!bat_CAMS_capturefolder!\!bat_SIMULATION.sim_session!"

	echo.[!Date:~4! !Time!]  DEL /Q /s "!bat_CAMS_distrofolder!\!bat_SIMULATION.sim_session!"
	                         DEL /Q /s "!bat_CAMS_distrofolder!\!bat_SIMULATION.sim_session!"
	echo.[!Date:~4! !Time!]  RD  /Q /s "!bat_CAMS_distrofolder!\!bat_SIMULATION.sim_session!"
	                         RD  /Q /s "!bat_CAMS_distrofolder!\!bat_SIMULATION.sim_session!"

	echo.[!Date:~4! !Time!]  DEL /Q    "!bat_CAMS_distrofolder!\!bat_SIMULATION.sim_session!_!bat_firstcamera!.zip"
	                         DEL /Q    "!bat_CAMS_distrofolder!\!bat_SIMULATION.sim_session!_!bat_firstcamera!.zip"
							   
	set bat_returncode=0


:SIM_DELETE_END

	echo.[!Date:~4! !Time!]      RETURN SIM_DELETE            [exitcode=!bat_returncode!]>>"!bat_LogFile!"
	exit /b !bat_returncode!





rem =========================================================================================
rem   :SIM_CAPTURE 
rem   Called from within the :LAUNCHCAPTURE subroutine.  It copies the files
rem   from directory specified in the "bat_SIMULATION.sim_sourcedir" variable.
rem 
rem   Just in case the computer is set to the en_NZ locale, the date is parsed
rem   accordingly.  Otherwise, the en_US locale date parsing is used.
rem 
rem     [SIMULATION]
rem     simulate=true
rem     sim_session=2013_10_30_06_21_56
rem     sim_sourcedir=!bat_CAMS_directory!\Simulation\CapturedFiles\2013_10_30_06_21_56
rem
rem   The date parsing is necessary for 
rem =========================================================================================

:SIM_CAPTURE

	echo.[!Date:~4! !Time!]      CALL SIM_CAPTURE %*>>"!bat_LogFile!"

	rem --------------- Parse date for default locale [en_US]
	rem                 The bat_date variable is set to the 10/30/2013 format
	rem                 The bat_time variable is set to the 06:21:56.00 format.  Since we don't
	rem                 really have the fractions of second, we'll use zero, since it probably won't matter.

	set bat_date=!bat_SIMULATION.sim_session:~5,2!/!bat_SIMULATION.sim_session:~8,2!/!bat_SIMULATION.sim_session:~0,4!
	set bat_time=!bat_SIMULATION.sim_session:~11,2!:!bat_SIMULATION.sim_session:~14,2!:!bat_SIMULATION.sim_session:~17,2!.00
	set bat_yyyy=!bat_date:~6,4!
	set bat_mm=!bat_date:~0,2!
	set bat_dd=!bat_date:~3,2!
	set bat_hh=!bat_time:~0,2!
	set bat_min=!bat_time:~3,2!
	set bat_sec=!bat_time:~6,2!
	
	rem --------------- Pad the bat_msec to exactly 3 digits
	set bat_msec=000!bat_time:~-2!0
	set bat_msec=!bat_msec:~-3!
	
	
	rEM --------------- Process for en-NZ locale
	if /i "!bat_locale!" equ "en-NZ" (
		set bat_mm=!bat_date:~3,2!
		set bat_dd=!bat_date:~0,2!
	)

	rem --------------- Set variables compatible with CAMS
	rem                 Note: the bat_captureddatetime variable should have the same value as
	rem                       bat_SIMULATION.sim_session
	set bat_captureddate=!bat_yyyy!_!bat_mm!_!bat_dd!
	set bat_capturedtime=!bat_hh!_!bat_min!_!bat_sec!
	set bat_captureddatetime=!bat_captureddate!_!bat_capturedtime!
	
	rem --------------- Make the target directory
   	echo.[!date:~4! !time!]          md "!bat_CAMS_capturefolder!\!bat_SIMULATION.sim_session!">>"!bat_LogFile!"
                                         md "!bat_CAMS_capturefolder!\!!bat_SIMULATION.sim_session!"
							   
	rem Copy the simulation source files to the target dir
    	echo.[!date:~4! !time!]          copy /y "!bat_CAMS_directory!\Simulation\CapturedFiles\!bat_captureddatetime!" "!bat_CAMS_capturefolder!\!bat_captureddatetime!">>"!bat_LogFile!"
                                         copy /y "!bat_CAMS_directory!\Simulation\CapturedFiles\!bat_captureddatetime!" "!bat_CAMS_capturefolder!\!bat_captureddatetime!"
							   
	set bat_returncode=0


:SIM_CAPTURE_END

	echo.[!Date:~4! !Time!]      RETURN SIM_CAPTURE           [exitcode=!bat_returncode!]>>"!bat_LogFile!"
	exit /b !bat_returncode!


:HELP
	echo.%~0	Batch Script
	echo.Usage: %~0 "<CapturedFiles directory>"
	echo.
exit /p 99




Rem ===========================================================================================
Rem    End of batch program
Rem ===========================================================================================

:DONE

	rem --------------- List the final state of all the variables
	rem SET BAT_
		echo.
		echo.[!date:~4! !time!]      DONE                          [exitcode=!bat_returncode!]>>"!bat_LogFile!"
		echo.>>"!bat_LogFile!"
 
		echo.[!date:~4! !time!]      DONE                          [exitcode=!bat_returncode!]
		echo.[!date:~4! !time!]~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		echo.

	rem --------------- Set the return code so a calling program or script can get the errorlevel or return code. 
	rem                 This is the same as the return code from main() in a C program.

	exit /b !bat_returncode!



Rem ############################################################################################################################
Rem ############################################################################################################################
Rem ############################################################################################################################
Rem ############################################################################################################################
Rem ############################################################################################################################

